//--------------------------------------------------------------------------------------
//  The Header Guard thingie plugin for Eddie
//  (C) Alan Ellis 1998 (alan@cgsoftware.com)
//
//  Unauthorized duplication is restricted, therefore I am authorizing you to duplicate
// this. =)
//
//
//--------------------------------------------------------------------------------------


#include <Alert.h>
#include <Bitmap.h>
#include <ctype.h>

#include "Plugin.h"
#include "HeaderGuardBitmaps.h"

void GuardHeader(PluginInterface *pluginInterface);


PluginInterface::PluginResult 
PluginMain(PluginInterface::PluginCallSelector selector, 
	PluginInterface *pluginInterface)
{
	PluginInterface::PluginResult result = PluginInterface::kIgnored;

// This is defined in the project options under "More compiler options"
// remove the: '-Wp, -DDEBUG' to prevent the strings from spewing at the
// terminal every time you press a key!  
#ifdef DEBUG	
	fprintf(stderr, "Header Guard: PluginInterface::PluginResult PluginMain() called\n"
                    "              Selector = %d\n", selector);
#endif

	switch (selector)
	{

		case PluginInterface::kGetPluginVersion:
		
			pluginInterface->ReturnSupportedPluginVersion();
			pluginInterface->ReturnRequiredPluginVersion();
			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kInit:			
		
			// publish the editor primitives used by the shortcuts:
			pluginInterface->RegisterHandledPrimitiveCommand("HeaderGuard",
				"This automagically places header guards on your .h files in the editor");

			result = PluginInterface::kAccepted;

#ifdef DEBUG	
	fprintf(stderr, "Header Guard: case PluginInterface::kInit:\n");
#endif

			break;
		
		case PluginInterface::kWindowOpened:
			
			pluginInterface->SetupPluginButton(kButtonBits, kPressedButtonBits);

			result = PluginInterface::kAccepted;
			break;

		case PluginInterface::kButtonClick:
			
			GuardHeader(pluginInterface);
			
			result = PluginInterface::kAccepted;
			break;


		case PluginInterface::kPrimitiveInvoked:
		{
			// handle a primitive from a keyboard command
			const char *primitive = pluginInterface->CurrentPrimitive();
			
			// make sure our shortcut was called
			if (strcmp(primitive, "HeaderGuard") == 0)
			{
				// animate the button as if it was pressed by a mouse to give the user
				// a positive feedback
				pluginInterface->AnimateButtonClick();
				
				GuardHeader(pluginInterface);
			}
			
			result = PluginInterface::kAccepted;
			break;
		}
		// appease the compiler warning gods and fall through a bunch of cases
		case PluginInterface::kUnload      :
		case PluginInterface::kQuit        :
		case PluginInterface::kButtonPress :
		case PluginInterface::kButtonDraw  :
		case PluginInterface::kArgvPassed  :
		case PluginInterface::kPulse       :
		case PluginInterface::kDraw        :
		case PluginInterface::kMessage     :
		case PluginInterface::kMouseDown   :
		case PluginInterface::kKeyDown     :
		case PluginInterface::kWindowActivated      :  
		case PluginInterface::kWindowDeactivated    :
		case PluginInterface::kDocumentTextChanged  : 	
		case PluginInterface::kDocumentOpened       :
		case PluginInterface::kDocumentClosed       :
		case PluginInterface::kDocumentStateChanged :
		case PluginInterface::kDocumentActivated    :
		case PluginInterface::kWindowClosed         :
		case PluginInterface::kGlobalSettingsSaved  :
		case PluginInterface::kDocumentSettingsSaved:
		case PluginInterface::kGlobalPrefsShowing   :
		case PluginInterface::kSyntaxColoringOff    :
		case PluginInterface::kFindQuotedString     : 
			result = PluginInterface::kIgnored;
			break;
		// something is wrong/new/unhandled
		default:
			fprintf(stderr, "Header Guard: Wrong/New/Unhandled case, selector value: %d\n", selector);
			result = PluginInterface::kIgnored;
			break;
	}
	return result;
}

const char *
PluginHelp(void)
{
	return "Header Guard";
}
//---------------------------------------------------------------------
// alan's stuff
//---------------------------------------------------------------------
void GuardHeader(PluginInterface *pluginInterface)
{
	// Is this a header file?
	// If not bail
	if (pluginInterface->LanguageType() != kCH)
		if (pluginInterface->LanguageType() != kCHPlus)
		{
			fprintf(stderr, "Header Guard: Not a C/C++ header file according to Eddie.\n");
			return;
		}
		
	int32 SelStart,
	      SelEnd;
	
	// Save the selection so that we may reset it when we are done
	pluginInterface->GetSelection(&SelStart, &SelEnd);

	// Let's get the header name and mangle it to our liking ...
	int32 HeaderNameLength = strlen(pluginInterface->Window()->Title());
	char *HeaderDefine = new char[HeaderNameLength + 1];
	
	strcpy(HeaderDefine, pluginInterface->Window()->Title());
	
	// Standard '.' to '_' mangle, we also need to remember that BeOs supports
	// file names with spaces, so ' ' also to '_', and, of course, all uppercase
	for (int32 i = 0; i < HeaderNameLength; ++i)
	{
		HeaderDefine[i] = toupper(HeaderDefine[i]);
		
		if ((HeaderDefine[i] == '.') || (HeaderDefine[i] == ' '))
			HeaderDefine[i] = '_';
	}
	
	// Create the top portion of the header guard in the form:
	// #ifndef HEADERNAME_H<newline>
	// #define HEADERNAME_H<newline>
	// <newline><NULL>
	char *Guard = new char[(HeaderNameLength << 1) + 20];
	
	sprintf(Guard, "#ifndef %s\n#define %s\n\n", HeaderDefine, HeaderDefine);
	
	// Stuff everything at the beginning of the file
	pluginInterface->StartUndo(SelStart, SelEnd, "Header Guard");
	
	int32 GuardCharLength = strlen(Guard);
	//pluginInterface->Select(0,0);
	pluginInterface->CurrentUndoInsert(Guard, GuardCharLength, 0);
	
	delete[] Guard;
	
	// Adjust selection to account for our inserthing of text
	SelStart += GuardCharLength;
	SelEnd   += GuardCharLength;
	
	// Add the #endif at the end
	//pluginInterface->Select(pluginInterface->TextLength(), pluginInterface->TextLength());
	pluginInterface->CurrentUndoInsert("\n#endif\n\n", 9, pluginInterface->TextLength());
	
	pluginInterface->SealUndo(SelStart, SelEnd);
	
	// Reset the selection as we found it
	//pluginInterface->Select(SelStart, SelEnd);
	//pluginInterface->ScrollToSelection();	
}
//--------------------------------------------------------------------